from artiq.experiment import *
from urukul_drivers import UrukulCh
import time
import numpy as np

# TODO:
#     [ ] Revisar los tiempos
#     [ ] Ver por que da overflow intermitentemente en las corrids
#     [ ] Ver como esta guardando los resultados y guardar lo que falta
#     [ ] Cambiarle los parametros a los laseres cuando arranca el exp

class SingleLine(EnvExperiment):
    """SP Transition Emission Experiment"""
    def build(self):
        self.setattr_device("core")
        self.setattr_device("ccb")

        self.pmt = self.get_device("ttl0")
        self.pmt_state = self.get_device("ttl4")
        self.laserUV = UrukulCh(self, ch=0, freq=100.0, amp=1.0)
        self.laserIR = UrukulCh(self, ch=1, freq=100.0, amp=1.0)

        self.setattr_argument("no_measures",
                              NumberValue(100, min=1, ndecimals=0, step=1),
                              "Experiment params")

        self.setattr_argument(f"t_prepS",
                              NumberValue(10*us, unit='us', scale=us, min=1*us),
                             "Experiment params")

        self.setattr_argument(f"t_ion",
                              NumberValue(10*us, unit='us', scale=us, min=1*us),
                             "Experiment params")

        self.setattr_argument(f"t_readout",
                              NumberValue(5*us, unit='us', scale=us, min=1*us),
                             "Experiment params")

    @rpc
    def create_datasets(self):
        self.set_dataset("counts", np.full(self.no_measures, np.nan),
                            broadcast=True, archive=True)
        self.set_dataset("t_readout", self.t_readout, broadcast=False, archive=True)
        self.set_dataset("t_prep_S", self.t_prepS, broadcast=False, archive=True)
        self.set_dataset("t_prep_ion", self.t_ion, broadcast=False, archive=True)
        self.set_dataset("no_measures", self.no_measures, broadcast=False, archive=True)
        # TODO: Agregar forma de guardar los datos de los canales del Urukul.
        # o bien guardando todos aca, o armando un metodo apropiado en su controlador

    @rpc
    def create_applets(self):
        self.ccb.issue("create_applet", "cuentas",
                        "${artiq_applet}histogram "
                        "counts "
                        "--update-delay 0.2")


    @kernel
    def run(self):
        self.create_datasets()
        self.create_applets()
        self.init_kernel()
        delay(100*ms)

        self.prep_ion()
        for runN in range(self.no_measures):
            # Aprovecho el tiempo que necesito para extraer datos
            # par enfriar al ion con los dos laseres prendidos
            at_mu(self.core.get_rtio_counter_mu() + 6000)
            self.prep_S()
            counts = self.readout()
            self.mutate_dataset("counts", runN, counts)

        self.cleanup()

    @kernel
    def init_kernel(self):
        self.core.reset()
        self.pmt.input()
        self.pmt_state.output()
        self.laserIR.initialize_channel()
        self.laserUV.initialize_channel()
        self.pmt_state.off()
        self.core.wait_until_mu(now_mu())


    @kernel
    def prep_ion(self):
        """Preparo el ion prendiendo ambos laseres"""
        self.laserIR.on()
        self.laserUV.on()
        #delay(self.t_ion)

    @kernel
    def prep_S(self):
        """Preparo el estado S prendiendo solamente el laser IR"""
        self.laserIR.on()
        self.laserUV.off()
        delay(self.t_prepS)


    @kernel
    def readout(self):
        """Registro cuentas emitidas con el laser UV prendido"""
        self.laserIR.off()
        self.laserUV.on()
        
        # Prendo y apago la TTL para ver en el osc.

        here = self.pmt.gate_rising(self.t_readout)

        self.pmt_state.pulse(self.t_readout)
        cuentas = self.pmt.count(here)
        delay(1*us)
        self.prep_ion()
        return cuentas

    @kernel
    def cleanup(self):
        self.core.break_realtime()
        self.laserIR.off()
        self.laserUV.off()
        self.pmt_state.off()
